/**
  An R2Vector models an element of R^2 with the standard inner product 
  and perp operation. 
  @version 31 May
*/    
class R2Vector{
    
	/** Internal variables are public for easy, immediate access. */
    public double x, y;
    
    /** Constructs the zero vector. */
    public R2Vector(){} 
    
    /** Constructs the vector (x,y). */
    public R2Vector(double x, double y){ this.x = x; this.y = y; }

    /** Constructs the vector with x = coords[0], y = coords[1]. */
    public R2Vector(double[] coords){ x = coords[0]; y = coords[1]; }
    // Add error checking in case coord.length is not 2.
    
    /** Returns a reference to a copy of this R2Vector. */
    public R2Vector copy(){ return new R2Vector(x,y); }

    /** @return string representation. */
    public String toString(){ return "(" + x + "," + y + ")"; }
    
    /** Vector addition. Not the same as method add.  Does not modify this R2Vector.
        @return Vector sum of this and w. */
    public R2Vector plus(R2Vector w) { return new R2Vector(x + w.x, y + w.y); }
    
    /** Vector addition. Not the same as method plus. Modifies this R2Vector. */
    public void add(R2Vector w) { x = x + w.x;  y = y + w.y; }
    
    /** Vector subtraction. Not the same as method sub. Does not modify this R2Vector.
        @return Vector difference this minus w. */
    public R2Vector minus(R2Vector w) { return new R2Vector(x - w.x, y - w.y); }
    
    /** Vector subtraction. Not the same as method minus.  Modifies this R2Vector. */
    public void sub(R2Vector w) { x = x - w.x;  y = y - w.y; }
    
    /** Scalar multiplication. Not the same as method mult. Does not modify this R2Vector.
        @return Product of this and the scalar r. */
    public R2Vector times(double r) { return new R2Vector(r*x, r*y); }

    /** Scalar multiplication. Not the same as method times. Modifies this R2Vector. */
    public void mult(double r) { x = r*x; y = r*y; }

    /** Dot product (also inner product). */
    public double dot(R2Vector w){ return x*w.x + y*w.y; }

    /** @return length of this. Same as getLength. */
    public double length(){ return Math.sqrt(Math.abs(this.dot(this))); }
    
    /** @return length of this. Same as method length. */
    public double getLength(){ return length(); }
    
    /** Perp operation: the perp of v is the result of rotating v
        counter clockwise by 90 degrees. */
    public R2Vector perp() { return new R2Vector(-y,x); }

	/** Direction vector.
		@return Unit vector determined by the angle theta. */
	public static R2Vector dirVec(double theta) 
		{ return new R2Vector(Math.cos(theta), Math.sin(theta)); }

	/** Angle of a vector. 
		@return Angle of the vector. */
	public static double angle(R2Vector v) { return Math.atan2(v.y,v.x); }
        
}
